@@ -1,12 +1,13 @@
 ## ----setup, include = FALSE---------------------------------------------------
 knitr::opts_chunk$set(echo = TRUE)
+knitr::opts_chunk$set(R.options = list(width = 100))
 
 library(EHR)
 library(lubridate)
 library(pkdata)
 options(stringsAsFactors = FALSE)
 
-## ----niceroutput, echo = FALSE------------------------------------------------
+## ----niceroutput, echo = FALSE--------------------------------------------------------------------
 findbreaks <- function(x, char = '[ /\\]', charlen = 75) {
   if(length(x) > 1) {
     out <- vapply(x, findbreaks, character(1), char, charlen, USE.NAMES = FALSE)
@@ -39,19 +40,20 @@ co <- function(expr) {
   cat("\n")
 }
 
-## ----load-lib-dir, eval=FALSE-------------------------------------------------
+## ----load-lib-dir, eval=FALSE---------------------------------------------------------------------
 #  # load EHR package and dependencies
 #  library(EHR)
 #  library(pkdata)
 #  library(lubridate)
 
-## ----simp-in------------------------------------------------------------------
+## ----simp-in--------------------------------------------------------------------------------------
 # define directories
 td <- tempdir()
-dir.create(file.path(td, 'check1'))
-rawDataDir <- system.file("examples", "str_ex1", package="EHR")
 checkDir <- file.path(td, 'check1')
+rawDataDir <- system.file("examples", "str_ex1", package="EHR")
+dir.create(checkDir)
 
+# pre-processed demographic data 
 demo <- read.csv(file.path(rawDataDir,"Demographics_DATA_simple.csv"))
 head(demo)
 
@@ -64,60 +66,62 @@ head(ivdose.data)
 creat.data <- read.csv(file.path(rawDataDir,"Creatinine_DATA_simple.csv"))
 head(creat.data)
 
-## ----simp-rename--------------------------------------------------------------
+## ----simp-rename----------------------------------------------------------------------------------
 names(conc.data)[1:2] <- names(demo)[1:2] <- c("mod_id", "mod_id_visit")
 names(creat.data)[1] <- names(ivdose.data)[1] <- "mod_id"
 
-## ----sim-build-pk-iv, eval = FALSE--------------------------------------------
+## ----sim-build-pk-iv, eval = FALSE----------------------------------------------------------------
 #  simple_pk_dat <- run_Build_PK_IV(
 #      conc=conc.data,
+#      conc.columns = list(id = 'mod_id', datetime = 'date.time', druglevel = 'conc.level',
+#                          idvisit = 'mod_id_visit'),
 #      dose=ivdose.data,
-#      lab.dat = list(creat.data),
-#      lab.vars = c('creat'),
+#      dose.columns = list(id = 'mod_id', date = 'date.dose', infuseDatetime = 'infuse.time',
+#                          infuseDose = 'infuse.dose', infuseTimeExact= 'infuse.time.real',
+#                          bolusDatetime = 'bolus.time', bolusDose = 'bolus.dose',
+#                          gap = 'maxint', weight = 'weight'),
 #      demo.list = demo,
-#      demo.vars=c('weight', 'weight_demo', 'height', 'gender', 'ageatsurgery',
-#                  'stat_sts', 'cpb_sts', 'length_of_icu_stay'),
-#      demo.abbr=c('wgt', 'wgt_demo', 'height', 'gender',
-#                  'age', 'stat', 'cpb', 'loi'),
-#      pk.vars=c('mod_id_visit', 'time', 'conc', 'dose', 'rate', 'event',
-#                'other', 'multiple.record', 'date', 'mod_id'),
+#      demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'),
+#      lab.list = list(creat.data),
+#      lab.columns = list(id = 'mod_id', datetime = 'date.time'),
 #      drugname='fent',
 #      check.path=checkDir)
 
-## ----sim-build-pk-iv-fake, echo = FALSE---------------------------------------
+## ----sim-build-pk-iv-fake, echo = FALSE-----------------------------------------------------------
 co({
 simple_pk_dat <- run_Build_PK_IV(
     conc=conc.data,
+    conc.columns = list(id = 'mod_id', datetime = 'date.time', druglevel = 'conc.level', 
+                        idvisit = 'mod_id_visit'),
     dose=ivdose.data,
-    lab.dat = list(creat.data),
-    lab.vars = c('creat'),
+    dose.columns = list(id = 'mod_id', date = 'date.dose', infuseDatetime = 'infuse.time', 
+                        infuseDose = 'infuse.dose', infuseTimeExact= 'infuse.time.real', 
+                        bolusDatetime = 'bolus.time', bolusDose = 'bolus.dose', 
+                        gap = 'maxint', weight = 'weight'),
     demo.list = demo,
-    demo.vars=c('weight', 'weight_demo', 'height', 'gender', 'ageatsurgery',
-                'stat_sts', 'cpb_sts', 'length_of_icu_stay'),
-    demo.abbr=c('wgt', 'wgt_demo', 'height', 'gender',
-                'age', 'stat', 'cpb', 'loi'),
-    pk.vars=c('mod_id_visit', 'time', 'conc', 'dose', 'rate', 'event',
-              'other', 'multiple.record', 'date', 'mod_id'),
+    demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'),
+    lab.list = list(creat.data),
+    lab.columns = list(id = 'mod_id', datetime = 'date.time'),
     drugname='fent',
     check.path=checkDir)
 })
 
-## ----sim-build-pk-iv-out------------------------------------------------------
+## ----sim-build-pk-iv-out--------------------------------------------------------------------------
 head(simple_pk_dat,15)
 
-## ----ex2-dirs-----------------------------------------------------------------
-dir.create(file.path(td, 'data2'))
-dir.create(file.path(td, 'check2'))
-rawDataDir <- system.file("examples", "str_ex2", package="EHR")
+## ----ex2-dirs-------------------------------------------------------------------------------------
 dataDir <- file.path(td, 'data2')
 checkDir <- file.path(td, 'check2')
+rawDataDir <- system.file("examples", "str_ex2", package="EHR")
+dir.create(dataDir)
+dir.create(checkDir)
 
-## ----demo-in------------------------------------------------------------------
+## ----demo-in--------------------------------------------------------------------------------------
 # demographics data
 demo.in <- readTransform(file.path(rawDataDir, "Demographics_DATA.csv"))
 head(demo.in)
 
-## ----samp-in1-----------------------------------------------------------------
+## ----samp-in1-------------------------------------------------------------------------------------
 # concentration sampling times data
 # read in raw data
 samp.raw <- read.csv(file.path(rawDataDir, "SampleTimes_DATA.csv"))
@@ -129,17 +133,17 @@ samp.in0 <- dataTransformation(samp.raw,
     modify = list(samp = expression(as.numeric(sub('Sample ', '', Event.Name)))))
 head(samp.in0)
 
-## ----samp-in2-----------------------------------------------------------------
+## ----samp-in2-------------------------------------------------------------------------------------
 # read in and transform data
 samp.in <- readTransform(file.path(rawDataDir, "SampleTimes_DATA.csv"),
     rename = c('Study.ID' = 'subject_id'),
     modify = list(samp = expression(as.numeric(sub('Sample ', '', Event.Name)))))
 head(samp.in)
 
-## ----conc-in1-----------------------------------------------------------------
+## ----conc-in1-------------------------------------------------------------------------------------
 # concentration sample values data
 # read in raw data
-conc.raw<-read.csv(file.path(rawDataDir, "SampleConcentration_DATA.csv"))
+conc.raw <-read.csv(file.path(rawDataDir, "SampleConcentration_DATA.csv"))
 head(conc.raw)
 
 # helper function used to make subject_id
@@ -163,7 +167,7 @@ conc.in0 <- dataTransformation(conc.raw,
                   )
 head(conc.in0)
 
-## ----conc-in2-----------------------------------------------------------------
+## ----conc-in2-------------------------------------------------------------------------------------
 # equivalent using readTransform()
 conc.in <- readTransform(file.path(rawDataDir, "SampleConcentration_DATA.csv"),
   modify = list(
@@ -177,20 +181,39 @@ conc.in <- readTransform(file.path(rawDataDir, "SampleConcentration_DATA.csv"),
   )
 head(conc.in)
 
-## ----flow-in------------------------------------------------------------------
+## ----flow-in--------------------------------------------------------------------------------------
+# FLOW dosing data
+flow.in <- readTransform(file.path(rawDataDir, "FLOW_DATA.csv"),
+ rename = c('Subject.Id' = 'subject_id',
+            'Subject.Uniq.Id' = 'subject_uid'),
+ modify=list(
+  date.time = expression(pkdata::parse_dates(EHR:::fixDates(Perform.Date))),
+  unit = expression(sub('.*[ ]', '', Final.Rate..NFR.units.)),
+  rate = expression(as.numeric(sub('([0-9.]+).*', '\\1', Final.Rate..NFR.units.)))
+  )
+ ) 
+head(flow.in)
+
+## ----flow-in0-------------------------------------------------------------------------------------
 # FLOW dosing data
 flow.in <- readTransform(file.path(rawDataDir, "FLOW_DATA.csv"),
                          rename = c('Subject.Id' = 'subject_id',
                                     'Subject.Uniq.Id' = 'subject_uid')) 
+# pre-process the flow data 
+# date.time variable should be in an appropriate form
+flow.in[,'date.time'] <- pkdata::parse_dates(EHR:::fixDates(flow.in[,'Perform.Date']))
+# unit and rate are required: separate unit and rate from 'Final.Rate..NFR.units.' if needed
+flow.in[,'unit'] <- sub('.*[ ]', '', flow.in[,'Final.Rate..NFR.units.'])
+flow.in[,'rate'] <- as.numeric(sub('([0-9.]+).*', '\\1', flow.in[,'Final.Rate..NFR.units.']))
 head(flow.in)
 
-## ----mar-in-------------------------------------------------------------------
+## ----mar-in---------------------------------------------------------------------------------------
 # MAR dosing data
 mar.in0 <- read.csv(file.path(rawDataDir, "MAR_DATA.csv"), check.names = FALSE)
 mar.in <- dataTransformation(mar.in0, rename = c('Uniq.Id' = 'subject_uid'))
 head(mar.in)
 
-## ----labs-in------------------------------------------------------------------
+## ----labs-in--------------------------------------------------------------------------------------
 # Serum creatinine lab data
 creat.in <- readTransform(file.path(rawDataDir, "Creatinine_DATA.csv"),
     rename = c('Subject.uniq' = 'subject_uid'))
@@ -199,9 +222,9 @@ head(creat.in)
 # Albumin lab data
 alb.in <- readTransform(file.path(rawDataDir, "Albumin_DATA.csv"),
     rename = c('Subject.uniq' = 'subject_uid'))
-head(creat.in)
+head(alb.in)
 
-## ----merge-ids----------------------------------------------------------------
+## ----merge-ids------------------------------------------------------------------------------------
 # merge all ID datasets
 data <-  list(demo.in,
               samp.in,
@@ -224,10 +247,10 @@ saveRDS(mod.id, file=file.path(dataDir,"Fentanyl_module_id.rds"))
 
 mod.id
 
-## ---- eval = FALSE------------------------------------------------------------
+## ---- eval = FALSE--------------------------------------------------------------------------------
 #  pullFakeId(dat, xwalk, firstCols = NULL, orderBy = NULL)
 
-## ----mod-id-data--------------------------------------------------------------
+## ----mod-id-data----------------------------------------------------------------------------------
 ## demographics data
 demo.cln <- pullFakeId(demo.in, mod.id,
     firstCols = c('mod_id', 'mod_visit', 'mod_id_visit'),
@@ -275,7 +298,7 @@ head(alb.cln)
 saveRDS(creat.cln, file=file.path(dataDir,"Fentanyl_creat_mod_id.rds"))
 saveRDS(alb.cln, file=file.path(dataDir,"Fentanyl_alb_mod_id.rds"))
 
-## ----mod-setup----------------------------------------------------------------
+## ----mod-setup------------------------------------------------------------------------------------
 # set crosswalk option 
 xwalk <- readRDS(file.path(dataDir, "Fentanyl_module_id.rds"))
 options(pkxwalk = 'xwalk')
@@ -284,7 +307,7 @@ options(pkxwalk = 'xwalk')
 drugname <- 'fent'
 LLOQ <- 0.05
 
-## ----Pro-Demographic, eval=TRUE-----------------------------------------------
+## ----Pro-Demographic, eval=TRUE-------------------------------------------------------------------
 # helper function
 exclude_val <- function(x, val=1) { !is.na(x) & x == val }
 
@@ -296,66 +319,73 @@ demo.out <- run_Demo(demo.path = file.path(dataDir, "Fentanyl_demo_mod_id.rds"),
 head(demo.out$demo)
 demo.out$exclude
 
-## ----Pro-Med-Str1, eval=FALSE-------------------------------------------------
-#  ivdose.out <- run_MedStrI(flow.path=file.path(dataDir,"Fentanyl_flow_mod_id.rds"),
-#      flow.select = c('mod_id','mod_id_visit','Perform.Date','Final.Wt..kg.',
-#                      'Final.Rate..NFR.units.','Final.Units'),
-#      flow.rename = c('mod_id','mod_id_visit', 'Perform.Date', 'weight',
-#                      'rate', 'final.units'),
-#      flow.mod.list = list(
-#        date.time = expression(parse_dates(fixDates(Perform.Date))),
-#        unit = expression(sub('.*[ ]', '', rate)),
-#        rate = expression(as.numeric(sub('([0-9.]+).*', '\\1', rate)))),
-#      medchk.path=file.path(rawDataDir, sprintf('medChecked-%s.csv', drugname)),
+## ----Pro-Med-Str1, eval=FALSE---------------------------------------------------------------------
+#  ivdose.out <- run_MedStrI(
 #      mar.path=file.path(dataDir,"Fentanyl_mar_mod_id.rds"),
-#      demo.list=demo.out,
-#      check.path=checkDir,
+#      mar.columns = list(id = 'mod_id', datetime = c('Date','Time'), dose = 'med:dosage',
+#                         drug = 'med:mDrug', given = 'med:given'),
+#      medGivenReq = TRUE,
+#      flow.path=file.path(dataDir,"Fentanyl_flow_mod_id.rds"),
+#      flow.columns = list(id = 'mod_id', datetime = 'date.time', finalunits = 'Final.Units',
+#                          unit = 'unit', rate = 'rate', weight = 'Final.Wt..kg.'),
+#      medchk.path = file.path(rawDataDir, sprintf('medChecked-%s.csv', drugname)),
+#      demo.list = NULL,
+#      demo.columns = list(),
+#      missing.wgt.path = NULL,
+#      wgt.columns = list(),
+#      check.path = checkDir,
 #      failflow_fn = 'FailFlow',
 #      failunit_fn = 'Unit',
 #      failnowgt_fn = 'NoWgt',
 #      infusion.unit = 'mcg/kg/hr',
 #      bolus.unit = 'mcg',
 #      bol.rate.thresh = Inf,
+#      rateunit = 'mcg/hr',
+#      ratewgtunit = 'mcg/kg/hr',
+#      weightunit = 'kg',
 #      drugname = drugname)
 
-## ----Pro-Med-Str1-fake, echo=FALSE--------------------------------------------
+## ----Pro-Med-Str1-fake, echo=FALSE----------------------------------------------------------------
 co({
-ivdose.out <- run_MedStrI(flow.path=file.path(dataDir,"Fentanyl_flow_mod_id.rds"), 
-    flow.select = c('mod_id','mod_id_visit','Perform.Date','Final.Wt..kg.',
-                    'Final.Rate..NFR.units.','Final.Units'),
-    flow.rename = c('mod_id','mod_id_visit', 'Perform.Date', 'weight',
-                    'rate', 'final.units'),
-    flow.mod.list = list(
-      date.time = expression(parse_dates(fixDates(Perform.Date))),
-      unit = expression(sub('.*[ ]', '', rate)),
-      rate = expression(as.numeric(sub('([0-9.]+).*', '\\1', rate)))),
-    medchk.path=file.path(rawDataDir, sprintf('medChecked-%s.csv', drugname)), 
+ivdose.out <- run_MedStrI(
     mar.path=file.path(dataDir,"Fentanyl_mar_mod_id.rds"),
-    demo.list=demo.out,
-    check.path=checkDir, 
+    mar.columns = list(id = 'mod_id', datetime = c('Date','Time'), dose = 'med:dosage', drug = 'med:mDrug', given = 'med:given'),
+    medGivenReq = TRUE,
+    flow.path=file.path(dataDir,"Fentanyl_flow_mod_id.rds"),
+    flow.columns = list(id = 'mod_id', datetime = 'date.time', finalunits = 'Final.Units', unit = 'unit', rate = 'rate', weight = 'Final.Wt..kg.'),
+    medchk.path = file.path(rawDataDir, sprintf('medChecked-%s.csv', drugname)),
+    demo.list = NULL,
+    demo.columns = list(),
+    missing.wgt.path = NULL,
+    wgt.columns = list(),
+    check.path = checkDir,
     failflow_fn = 'FailFlow',
     failunit_fn = 'Unit',
     failnowgt_fn = 'NoWgt',
     infusion.unit = 'mcg/kg/hr',
     bolus.unit = 'mcg',
     bol.rate.thresh = Inf,
+    rateunit = 'mcg/hr',
+    ratewgtunit = 'mcg/kg/hr',
+    weightunit = 'kg',
     drugname = drugname)
     })
 
-## ----Pro-Med-Str1-out---------------------------------------------------------
+## ----Pro-Med-Str1-out-----------------------------------------------------------------------------
 head(ivdose.out)
 
-## ----eRX-dat------------------------------------------------------------------
+## ----eRX-dat--------------------------------------------------------------------------------------
 (eRX <- read.csv(file.path(rawDataDir,"e-rx_DATA.csv"),stringsAsFactors = FALSE))
 
-## ----Pro-Med-Str2-------------------------------------------------------------
+## ----Pro-Med-Str2---------------------------------------------------------------------------------
 eRX.out <- run_MedStrII(file.path(rawDataDir,"e-rx_DATA.csv"),
-    select = c('GRID','MED_NAME','RX_DOSE','FREQUENCY','ENTRY_DATE','STRENGTH_AMOUNT','DESCRIPTION'),
-    rename = c('ID','MED_NAME','RX_DOSE','FREQUENCY','ENTRY_DATE','STRENGTH_AMOUNT','DESCRIPTION'))
+    dat.columns = list(id = 'GRID', dose = 'RX_DOSE', freq = 'FREQUENCY', date = 'ENTRY_DATE', 
+                       str = 'STRENGTH_AMOUNT', desc = 'DESCRIPTION')
+)
 
 eRX.out
 
-## ----Pro-Drug-Level, eval=FALSE-----------------------------------------------
+## ----Pro-Drug-Level, eval=FALSE-------------------------------------------------------------------
 #  conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
 #      conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
 #      conc.rename=c(fentanyl_calc_conc = 'conc.level', samp= 'event'),
@@ -369,9 +399,8 @@ eRX.out
 #      drugname=drugname,
 #      LLOQ=LLOQ,
 #      demo.list=demo.out)
-#  head(conc.out)
 
-## ----Pro-Drug-Level-fake, echo=FALSE------------------------------------------
+## ----Pro-Drug-Level-fake, echo=FALSE--------------------------------------------------------------
 co({
 conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
     conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
@@ -388,22 +417,22 @@ conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"
     demo.list=demo.out)
 })
 
-## ----Pro-Drug-Level-out-------------------------------------------------------
+## ----Pro-Drug-Level-out---------------------------------------------------------------------------
 head(conc.out)
 
-## ----faildate-----------------------------------------------------------------
+## ----faildate-------------------------------------------------------------------------------------
 ( fail.miss.conc.date <- read.csv(file.path(checkDir,"failMissingConcDate-fent.csv")) )
 
-## ----fixdate------------------------------------------------------------------
+## ----fixdate--------------------------------------------------------------------------------------
 fail.miss.conc.date[,"date.time"] <- c("9/30/2016 09:32","10/1/2016 19:20","10/2/2016 02:04")
 fail.miss.conc.date
  
 write.csv(fail.miss.conc.date, file.path(checkDir,"fixMissingConcDate-fent.csv"))
 
-## ----Pro-Drug-Level-rerun, eval=FALSE-----------------------------------------
+## ----Pro-Drug-Level-rerun, eval=FALSE-------------------------------------------------------------
 #  conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
 #      conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
-#      conc.rename=c(fentanyl_calc_conc = 'conc.level', samp= 'event'),
+#      conc.rename=c(fentanyl_calc_conc = 'conc.level', samp = 'event'),
 #      conc.mod.list=list(mod_id_event = expression(paste(mod_id_visit, event, sep = '_'))),
 #      samp.path=file.path(dataDir,"Fentanyl_samp_mod_id.rds"),
 #      samp.mod.list=list(mod_id_event = expression(paste(mod_id_visit, samp, sep = '_'))),
@@ -415,11 +444,11 @@ write.csv(fail.miss.conc.date, file.path(checkDir,"fixMissingConcDate-fent.csv")
 #      LLOQ=LLOQ,
 #      demo.list=demo.out)
 
-## ----Pro-Drug-Level-rerun-fake, echo=FALSE------------------------------------
+## ----Pro-Drug-Level-rerun-fake, echo=FALSE--------------------------------------------------------
 co({
 conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"),
     conc.select=c('mod_id','mod_id_visit','samp','fentanyl_calc_conc'),
-    conc.rename=c(fentanyl_calc_conc = 'conc.level', samp= 'event'),
+    conc.rename=c(fentanyl_calc_conc = 'conc.level', samp = 'event'),
     conc.mod.list=list(mod_id_event = expression(paste(mod_id_visit, event, sep = '_'))),
     samp.path=file.path(dataDir,"Fentanyl_samp_mod_id.rds"),
     samp.mod.list=list(mod_id_event = expression(paste(mod_id_visit, samp, sep = '_'))),
@@ -432,7 +461,7 @@ conc.out <- run_DrugLevel(conc.path=file.path(dataDir,"Fentanyl_conc_mod_id.rds"
     demo.list=demo.out)
 })
 
-## ----remove-fix, include=FALSE------------------------------------------------
+## ----remove-fix, include=FALSE--------------------------------------------------------------------
 # remove fix file, so running vignette produces warning with first run of run_DrugLevel()
 fx <- file.path(checkDir,"fixMissingConcDate-fent.csv")
 if (file.exists(fx)) file.remove(fx)
@@ -441,7 +470,7 @@ if (file.exists(fx)) file.remove(fx)
 ms <- file.path(checkDir,paste0("multipleSetsConc-", drugname, Sys.Date(),".csv"))
 if (file.exists(ms)) file.remove(ms)
 
-## ----Pro-Laboratory, eval=TRUE------------------------------------------------
+## ----Pro-Laboratory, eval=TRUE--------------------------------------------------------------------
 creat.out <- run_Labs(lab.path=file.path(dataDir,"Fentanyl_creat_mod_id.rds"),
     lab.select = c('mod_id','date.time','creat'),
     lab.mod.list = list(date.time = expression(parse_dates(fixDates(paste(date, time))))))
@@ -454,19 +483,21 @@ lab.out <- list(creat.out, alb.out)
 
 str(lab.out)
 
-## ----Build-PK-IV, eval=FALSE--------------------------------------------------
-#  pk_dat <- run_Build_PK_IV(conc=conc.out,
+## ----Build-PK-IV, eval=FALSE----------------------------------------------------------------------
+#  pk_dat <- run_Build_PK_IV(
+#      conc=conc.out,
+#      conc.columns = list(id = 'mod_id', datetime = 'date.time', druglevel = 'conc.level',
+#                          idvisit = 'mod_id_visit'),
 #      dose=ivdose.out,
-#      demo.list=demo.out,
-#      demo.vars=c('weight', 'weight_demo', 'height', 'gender',
-#                  'ageatsurgery', 'stat_sts', 'cpb_sts',
-#                  'length_of_icu_stay'),
-#      demo.abbr=c('wgt', 'wgt_demo', 'height', 'gender',
-#                  'age', 'stat', 'cpb', 'loi'),
-#      lab.dat = lab.out,
-#      lab.vars = c('creat','alb'),
-#      pk.vars=c('mod_id_visit', 'time', 'conc', 'dose', 'rate', 'event',
-#                'other', 'multiple.record', 'date', 'mod_id'),
+#      dose.columns = list(id = 'mod_id', date = 'date.dose', infuseDatetime = 'infuse.time',
+#                          infuseDose = 'infuse.dose', infuseTimeExact= 'infuse.time.real',
+#                          bolusDatetime = 'bolus.time', bolusDose = 'bolus.dose',
+#                          gap = 'maxint', weight = 'weight'),
+#      demo.list = demo.out,
+#      demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'),
+#      lab.list = lab.out,
+#      lab.columns = list(id = 'mod_id', datetime = 'date.time'),
+#      pk.vars=c('date'),
 #      drugname=drugname,
 #      check.path=checkDir,
 #      missdemo_fn='-missing-demo',
@@ -474,20 +505,20 @@ str(lab.out)
 #      date.format="%m/%d/%y %H:%M:%S",
 #      date.tz="America/Chicago")
 
-## ----Build-PK-IV-fake, echo=FALSE---------------------------------------------
+## ----Build-PK-IV-fake, echo=FALSE-----------------------------------------------------------------
 co({
-pk_dat <- run_Build_PK_IV(conc=conc.out,
+pk_dat <- run_Build_PK_IV(
+    conc=conc.out,
+    conc.columns = list(id = 'mod_id', datetime = 'date.time', druglevel = 'conc.level', 
+                        idvisit = 'mod_id_visit'),
     dose=ivdose.out,
-    demo.list=demo.out,
-    demo.vars=c('weight', 'weight_demo', 'height', 'gender',
-                'ageatsurgery', 'stat_sts', 'cpb_sts',
-                'length_of_icu_stay'),
-    demo.abbr=c('wgt', 'wgt_demo', 'height', 'gender',
-                'age', 'stat', 'cpb', 'loi'),
-    lab.dat = lab.out,
-    lab.vars = c('creat','alb'),
-    pk.vars=c('mod_id_visit', 'time', 'conc', 'dose', 'rate', 'event',
-              'other', 'multiple.record', 'date', 'mod_id'),
+    dose.columns = list(id = 'mod_id', date = 'date.dose', infuseDatetime = 'infuse.time', 
+                        infuseDose = 'infuse.dose', infuseTimeExact= 'infuse.time.real',  bolusDatetime = 'bolus.time', bolusDose = 'bolus.dose', gap = 'maxint', weight = 'weight'),
+    demo.list = demo.out,
+    demo.columns = list(id = 'mod_id', idvisit = 'mod_id_visit'),
+    lab.list = lab.out,
+    lab.columns = list(id = 'mod_id', datetime = 'date.time'),
+    pk.vars=c('date'),
     drugname=drugname,
     check.path=checkDir,
     missdemo_fn='-missing-demo',
@@ -496,13 +527,13 @@ pk_dat <- run_Build_PK_IV(conc=conc.out,
     date.tz="America/Chicago")
 })
 
-## ----Build-PK-IV-out----------------------------------------------------------
+## ----Build-PK-IV-out------------------------------------------------------------------------------
 # convert id back to original IDs
 pk_dat <- pullRealId(pk_dat, remove.mod.id=TRUE)
 
 head(pk_dat)
 
-## ----cleanup, echo = FALSE----------------------------------------------------
+## ----cleanup, echo = FALSE------------------------------------------------------------------------
 # normally, you would not delete these files
 # CRAN policy states that a package should do proper cleanup
 to_delete <- c(file.path(td, 'check1'), file.path(td, 'check2'), file.path(td, 'data2'))
